Class: DependencyChart::LinkRules
- Defined in:
- lib/jirametrics/dependency_chart.rb
Instance Attribute Summary collapse
-
#label ⇒ Object
Returns the value of attribute label.
-
#line_color ⇒ Object
Returns the value of attribute line_color.
Instance Method Summary collapse
- #bidirectional_arrows? ⇒ Boolean
-
#get_merge_bidirectional ⇒ Object
get_ prefix because the plain name is the config DSL setter (merge_bidirectional keep:).
- #merge_bidirectional(keep: 'inward') ⇒ Object
- #use_bidirectional_arrows ⇒ Object
Methods inherited from Rules
Instance Attribute Details
#label ⇒ Object
Returns the value of attribute label.
9 10 11 |
# File 'lib/jirametrics/dependency_chart.rb', line 9 def label @label end |
#line_color ⇒ Object
Returns the value of attribute line_color.
9 10 11 |
# File 'lib/jirametrics/dependency_chart.rb', line 9 def line_color @line_color end |
Instance Method Details
#bidirectional_arrows? ⇒ Boolean
26 27 28 |
# File 'lib/jirametrics/dependency_chart.rb', line 26 def bidirectional_arrows? @use_bidirectional_arrows end |
#get_merge_bidirectional ⇒ Object
get_ prefix because the plain name is the config DSL setter (merge_bidirectional keep:).
18 19 20 |
# File 'lib/jirametrics/dependency_chart.rb', line 18 def get_merge_bidirectional # rubocop:disable Naming/AccessorMethodName @merge_bidirectional end |
#merge_bidirectional(keep: 'inward') ⇒ Object
11 12 13 14 15 |
# File 'lib/jirametrics/dependency_chart.rb', line 11 def merge_bidirectional keep: 'inward' raise "Keep must be either inward or outward: #{keep}" unless %i[inward outward].include? keep.to_sym @merge_bidirectional = keep.to_sym end |
#use_bidirectional_arrows ⇒ Object
22 23 24 |
# File 'lib/jirametrics/dependency_chart.rb', line 22 def use_bidirectional_arrows @use_bidirectional_arrows = true end |