Class: Issue::TrackedSprint

Inherits:
Data
  • Object
show all
Defined in:
lib/jirametrics/issue.rb

Overview

Like SprintMembership but also tracks when this issue was added, used while pairing sprint entries and exits in #sprint_entry_events.

Instance Attribute Summary collapse

Instance Attribute Details

#add_timeObject (readonly)

Returns the value of attribute add_time

Returns:

  • (Object)

    the current value of add_time



242
243
244
# File 'lib/jirametrics/issue.rb', line 242

def add_time
  @add_time
end

#changeObject (readonly)

Returns the value of attribute change

Returns:

  • (Object)

    the current value of change



242
243
244
# File 'lib/jirametrics/issue.rb', line 242

def change
  @change
end

#sprint_idObject (readonly)

Returns the value of attribute sprint_id

Returns:

  • (Object)

    the current value of sprint_id



242
243
244
# File 'lib/jirametrics/issue.rb', line 242

def sprint_id
  @sprint_id
end

#sprint_startObject (readonly)

Returns the value of attribute sprint_start

Returns:

  • (Object)

    the current value of sprint_start



242
243
244
# File 'lib/jirametrics/issue.rb', line 242

def sprint_start
  @sprint_start
end