Class: AgingWorkInProgressChart
- Includes:
- GroupableIssueChart
- Defined in:
- lib/jirametrics/aging_work_in_progress_chart.rb
Instance Attribute Summary collapse
-
#board_columns ⇒ Object
readonly
Returns the value of attribute board_columns.
-
#board_id ⇒ Object
Returns the value of attribute board_id.
-
#possible_statuses ⇒ Object
Returns the value of attribute possible_statuses.
Attributes inherited from ChartBase
#aggregated_project, #all_boards, #canvas_height, #canvas_width, #data_quality, #date_range, #file_system, #holiday_dates, #issues, #settings, #time_range, #timezone_offset
Instance Method Summary collapse
- #adjust_chart_height ⇒ Object
- #adjust_visibility_of_unmapped_status_column(data_sets:) ⇒ Object
- #column_for(issue:) ⇒ Object
- #determine_board_columns ⇒ Object
- #indexes_of_leading_and_trailing_zeros(list) ⇒ Object
-
#initialize(block) ⇒ AgingWorkInProgressChart
constructor
A new instance of AgingWorkInProgressChart.
- #make_data_sets ⇒ Object
- #percentiles(percentile_color_hash) ⇒ Object
- #run ⇒ Object
-
#show_all_columns(show = true) ⇒ Object
rubocop:disable Style/OptionalBooleanParameter.
Methods included from GroupableIssueChart
#group_issues, #grouping_rules, #init_configuration_block
Methods inherited from ChartBase
#aggregated_project?, #canvas, #canvas_responsive?, #chart_format, #collapsible_issues_panel, #color_block, #color_for, #completed_issues_in_range, #current_board, #daily_chart_dataset, #describe_non_working_days, #description_text, #format_integer, #format_status, #header_text, #holidays, #html_directory, #icon_span, #label_days, #label_issues, #link_to_issue, #next_id, #random_color, #render, #render_top_text, #status_category_color, #working_days_annotation, #wrap_and_render
Constructor Details
#initialize(block) ⇒ AgingWorkInProgressChart
Returns a new instance of AgingWorkInProgressChart.
12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 12 def initialize block super() header_text 'Aging Work in Progress' description_text <<-HTML <p> This chart shows only work items that have started but not completed, grouped by the column they're currently in. Hovering over a dot will show you the ID of that work item. </p> <p> The shaded areas indicate what percentage of the work has passed that column within that time. Notes: <ul> <li>It only shows columns that are considered "in progress".</li> <li>If you see a colour group that drops as it moves to the right, that generally indicates that a different number of data points is being included in each column. Probably because tickets moved backwards athough it could also indicate that a ticket jumped over columns as it moved to the right. </li> </ul> </p> <div style="border: 1px solid gray; padding: 0.2em"> <% @percentiles.keys.sort.reverse.each do |percent| %> <span style="padding-left: 0.5em; padding-right: 0.5em; vertical-align: middle;"><%= color_block @percentiles[percent] %> <%= percent %>%</span> <% end %> </div> HTML percentiles( # 50 => '--aging-work-in-progress-chart-shading-50-color', 85 => '--aging-work-in-progress-chart-shading-85-color', # 98 => '--aging-work-in-progress-chart-shading-98-color', # 100 => '--aging-work-in-progress-chart-shading-100-color' ) show_all_columns false init_configuration_block(block) do grouping_rules do |issue, rule| rule.label = issue.type rule.color = color_for type: issue.type end end end |
Instance Attribute Details
#board_columns ⇒ Object (readonly)
Returns the value of attribute board_columns.
10 11 12 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 10 def board_columns @board_columns end |
#board_id ⇒ Object
Returns the value of attribute board_id.
9 10 11 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 9 def board_id @board_id end |
#possible_statuses ⇒ Object
Returns the value of attribute possible_statuses.
9 10 11 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 9 def possible_statuses @possible_statuses end |
Instance Method Details
#adjust_chart_height ⇒ Object
186 187 188 189 190 191 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 186 def adjust_chart_height min_height = @max_age * 5 @canvas_height = min_height if min_height > @canvas_height @canvas_height = 400 if min_height > 400 end |
#adjust_visibility_of_unmapped_status_column(data_sets:) ⇒ Object
164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 164 def adjust_visibility_of_unmapped_status_column data_sets: column_name = @fake_column.name has_unmapped = data_sets.any? do |set| set['data'].any? do |data| data['x'] == column_name if data.is_a? Hash end end if has_unmapped @description_text += "<p>The items shown in #{column_name.inspect} are not visible on the " \ 'board but are still active. Most likely everyone has forgotten about them.</p>' else # @column_headings.pop @board_columns.pop end end |
#column_for(issue:) ⇒ Object
158 159 160 161 162 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 158 def column_for issue: @board_columns.find do |board_column| board_column.status_ids.include? issue.status.id end end |
#determine_board_columns ⇒ Object
69 70 71 72 73 74 75 76 77 78 79 80 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 69 def determine_board_columns unmapped_statuses = current_board.possible_statuses.collect(&:id) columns = current_board.visible_columns columns.each { |c| unmapped_statuses -= c.status_ids } @fake_column = BoardColumn.new({ 'name' => '[Unmapped Statuses]', 'statuses' => unmapped_statuses.collect { |id| { 'id' => id.to_s } }.uniq # rubocop:disable Performance/ChainArrayAllocation }) @board_columns = columns + [@fake_column] end |
#indexes_of_leading_and_trailing_zeros(list) ⇒ Object
141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 141 def indexes_of_leading_and_trailing_zeros list result = [] 0.upto(list.size - 1) do |index| break unless list[index].zero? result << index end stop_at = result.empty? ? 0 : (result.last + 1) (list.size - 1).downto(stop_at).each do |index| break unless list[index].zero? result << index if list[index].zero? end result end |
#make_data_sets ⇒ Object
82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 82 def make_data_sets aging_issues = @issues.select do |issue| board = issue.board board.id == @board_id && board.cycletime.in_progress?(issue) end @max_age = 20 rules_to_issues = group_issues aging_issues data_sets = rules_to_issues.keys.collect do |rules| { 'type' => 'line', 'label' => rules.label, 'data' => rules_to_issues[rules].filter_map do |issue| age = issue.board.cycletime.age(issue, today: date_range.end) column = column_for issue: issue next if column.nil? @max_age = age if age > @max_age { 'y' => age, 'x' => column.name, 'title' => ["#{issue.key} : #{issue.summary} (#{label_days age})"] } end, 'fill' => false, 'showLine' => false, 'backgroundColor' => rules.color } end calculator = BoardMovementCalculator.new board: @all_boards[@board_id], issues: issues, today: date_range.end column_indexes_to_remove = [] unless @show_all_columns column_indexes_to_remove = indexes_of_leading_and_trailing_zeros(calculator.age_data_for(percentage: 100)) column_indexes_to_remove.reverse_each do |index| @board_columns.delete_at index end end calculator.stacked_age_data_for(percentages: @percentiles.keys).each do |percentage, data| column_indexes_to_remove.reverse_each { |index| data.delete_at index } color = @percentiles[percentage] data_sets << { 'type' => 'bar', 'label' => "#{percentage}%", 'barPercentage' => 1.0, 'categoryPercentage' => 1.0, 'backgroundColor' => color, 'data' => data } end data_sets end |
#percentiles(percentile_color_hash) ⇒ Object
182 183 184 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 182 def percentiles percentile_color_hash @percentiles = percentile_color_hash.transform_values { |value| CssVariable[value] } end |
#run ⇒ Object
53 54 55 56 57 58 59 60 61 62 63 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 53 def run determine_board_columns @header_text += " on board: #{@all_boards[@board_id].name}" data_sets = make_data_sets adjust_visibility_of_unmapped_status_column data_sets: data_sets adjust_chart_height wrap_and_render(binding, __FILE__) end |
#show_all_columns(show = true) ⇒ Object
rubocop:disable Style/OptionalBooleanParameter
65 66 67 |
# File 'lib/jirametrics/aging_work_in_progress_chart.rb', line 65 def show_all_columns show = true # rubocop:disable Style/OptionalBooleanParameter @show_all_columns = show end |