Class: AgingWorkTable
Constant Summary
Constants inherited
from ChartBase
ChartBase::LABEL_POSITIONS
Instance Attribute Summary collapse
Attributes inherited from ChartBase
#aggregated_project, #all_boards, #atlassian_document_format, #board_id, #canvas_height, #canvas_width, #data_quality, #date_range, #file_system, #fix_versions, #holiday_dates, #issues, #settings, #time_range, #timezone_offset, #x_axis_title, #y_axis_title
Instance Method Summary
collapse
Methods inherited from ChartBase
#aggregated_project?, #before_run, #call_before_run, #canvas, #canvas_responsive?, #chart_format, #collapsible_issues_panel, #color_block, #color_for, #completed_issues_in_range, #current_board, #cycletime, #cycletime_for_issue, #daily_chart_dataset, #date_annotation, #describe_non_working_days, #description_text, #format_integer, #format_status, #header_text, #holidays, #html_directory, #icon_span, #label_days, #label_hours, #label_issues, #label_minutes, #link_to_issue, #next_id, #normalize_annotation_datetime, #not_visible_text, #random_color, #render, #render_axis_title, #render_top_text, #seam_end, #seam_start, #stagger_label_positions, #status_category_color, #to_human_readable, #working_days_annotation, #wrap_and_render
Constructor Details
Returns a new instance of AgingWorkTable.
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
|
# File 'lib/jirametrics/aging_work_table.rb', line 9
def initialize block
super()
@stalled_threshold = 5
@dead_threshold = 45
@age_cutoff = 0
'Aging Work Table'
description_text <<-TEXT
<p>
This chart shows all active (started but not completed) work, ordered from oldest at the top to
newest at the bottom.
</p>
<p>
If there are expedited items that haven't yet started then they're at the bottom of the table.
By the very definition of expedited, if we haven't started them already, we'd better get on that.
</p>
<p>
Legend:
<ul>
<li><b>E:</b> Whether this item is <b>E</b>xpedited.</li>
<li><b>B/S:</b> Whether this item is either <b>B</b>locked or <b>S</b>talled.</li>
<li><b>Forecast:</b> A forecast of how long it is likely to take to finish this work item.</li>
</ul>
</p>
TEXT
instance_eval(&block)
end
|
Instance Attribute Details
#any_scrum_boards ⇒ Object
Returns the value of attribute any_scrum_boards.
7
8
9
|
# File 'lib/jirametrics/aging_work_table.rb', line 7
def any_scrum_boards
@any_scrum_boards
end
|
#today ⇒ Object
Returns the value of attribute today.
6
7
8
|
# File 'lib/jirametrics/aging_work_table.rb', line 6
def today
@today
end
|
Instance Method Details
#age_cutoff(age = nil) ⇒ Object
158
159
160
161
|
# File 'lib/jirametrics/aging_work_table.rb', line 158
def age_cutoff age = nil
@age_cutoff = age.to_i if age
@age_cutoff
end
|
#blocked_text(issue) ⇒ Object
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
|
# File 'lib/jirametrics/aging_work_table.rb', line 80
def blocked_text issue
started_time, _stopped_time = issue.started_stopped_times
return nil if started_time.nil?
current = issue.blocked_stalled_changes(end_time: time_range.end)[-1]
if current.blocked?
color_block '--blocked-color', title: current.reasons
elsif current.stalled?
if current.stalled_days && current.stalled_days > @dead_threshold
color_block(
'--dead-color',
title: "Dead? Hasn't had any activity in #{label_days current.stalled_days}. " \
'Does anyone still care about this?'
)
else
color_block '--stalled-color', title: current.reasons
end
end
end
|
#dates_text(issue) ⇒ Object
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
|
# File 'lib/jirametrics/aging_work_table.rb', line 123
def dates_text issue
date = date_range.end
due = issue.due_date
message = nil
calculator = @calculators[issue.board.id]
days_remaining, error = calculator.forecasted_days_remaining_and_message issue: issue, today: @today
unless error
if due
if due < date
message = "Due: <b>#{due}</b> (#{label_days (@today - due).to_i} ago)"
error = 'Overdue'
elsif due == date
message = 'Due: <b>today</b>'
else
error = 'Due date at risk' if date_range.end + days_remaining > due
message = "Due: <b>#{due}</b> (#{label_days (due - @today).to_i})"
end
else
"#{label_days days_remaining} left."
end
end
text = +''
text << "<span title='#{error}' style='color: red'>ⓘ </span>" if error
if days_remaining
text << "#{label_days days_remaining} left"
else
text << 'Unable to forecast'
end
text << ' | ' << message if message
text
end
|
#expedited_but_not_started ⇒ Object
53
54
55
56
57
58
|
# File 'lib/jirametrics/aging_work_table.rb', line 53
def expedited_but_not_started
@issues.select do |issue|
started_time, stopped_time = issue.started_stopped_times
started_time.nil? && stopped_time.nil? && issue.expedited?
end.sort_by(&:created)
end
|
#expedited_text(issue) ⇒ Object
73
74
75
76
77
78
|
# File 'lib/jirametrics/aging_work_table.rb', line 73
def expedited_text issue
return unless issue.expedited?
name = issue.raw['fields']['priority']['name']
color_block '--expedited-color', title: "Expedited: Has a priority of "#{name}""
end
|
#fix_versions_text(issue) ⇒ Object
100
101
102
103
104
105
106
107
108
109
|
# File 'lib/jirametrics/aging_work_table.rb', line 100
def fix_versions_text issue
issue.fix_versions.collect do |fix|
if fix.released?
icon_text = icon_span title: 'Released. Likely not on the board anymore.', icon: '✅'
"#{fix.name} #{icon_text}"
else
fix.name
end
end.join('<br />')
end
|
#initialize_calculator ⇒ Object
This is its own method simply so the tests can initialize the calculator without doing a full run.
46
47
48
49
50
51
|
# File 'lib/jirametrics/aging_work_table.rb', line 46
def initialize_calculator
@today = date_range.end
@calculators = @all_boards.transform_values do |board|
BoardMovementCalculator.new board: board, issues: issues, today: @today
end
end
|
#parent_hierarchy(issue) ⇒ Object
163
164
165
166
167
168
169
170
171
172
173
174
175
176
|
# File 'lib/jirametrics/aging_work_table.rb', line 163
def parent_hierarchy issue
result = []
while issue
cyclical_parent_links = result.include? issue
result << issue
break if cyclical_parent_links
issue = issue.parent
end
result.reverse
end
|
#priority_text(issue) ⇒ Object
178
179
180
|
# File 'lib/jirametrics/aging_work_table.rb', line 178
def priority_text issue
"<img src='#{issue.priority_url}' title='Priority: #{issue.priority_name}' style='max-width: 1em;'/>"
end
|
#run ⇒ Object
38
39
40
41
42
43
|
# File 'lib/jirametrics/aging_work_table.rb', line 38
def run
initialize_calculator
aging_issues = select_aging_issues + expedited_but_not_started
wrap_and_render(binding, __FILE__)
end
|
#select_aging_issues ⇒ Object
60
61
62
63
64
65
66
67
68
69
70
71
|
# File 'lib/jirametrics/aging_work_table.rb', line 60
def select_aging_issues
aging_issues = @issues.select do |issue|
started, stopped = issue.started_stopped_times
next false if started.nil? || stopped
next true if issue.blocked_on_date?(@today, end_time: time_range.end) || issue.expedited?
age = (@today - started.to_date).to_i + 1
age > @age_cutoff
end
@any_scrum_boards = aging_issues.any? { |issue| issue.board.scrum? }
aging_issues.sort { |a, b| b.board.cycletime.age(b, today: @today) <=> a.board.cycletime.age(a, today: @today) }
end
|
#sprints_text(issue) ⇒ Object
111
112
113
114
115
116
117
118
119
120
121
|
# File 'lib/jirametrics/aging_work_table.rb', line 111
def sprints_text issue
issue.sprints.collect do |sprint|
icon_text = nil
if sprint.active?
icon_text = icon_span title: 'Active sprint', icon: '➡️'
elsif sprint.closed?
icon_text = icon_span title: 'Sprint closed', icon: '✅'
end
"#{sprint.name} #{icon_text}"
end.join('<br />')
end
|