Class: DownloaderForCloud

Inherits:
Downloader show all
Defined in:
lib/jirametrics/downloader_for_cloud.rb

Constant Summary

Constants inherited from Downloader

Downloader::CURRENT_METADATA_VERSION

Instance Attribute Summary

Attributes inherited from Downloader

#board_id_to_filter_id, #file_system, #metadata, #start_date_in_query

Instance Method Summary collapse

Methods inherited from Downloader

create, #download_features, #download_github_prs, #download_sprints, #download_statuses, #download_users, #end_progress, #extract_project_keys_from_downloaded_issues, #file_prefix, #find_board_ids, #identify_other_issues_to_be_downloaded, #initialize, #load_metadata, #log, #log_start, #make_jql, #metadata_pathname, #progress_dot, #remove_old_files, #save_metadata, #start_progress, #timezone_offset, #today_in_project_timezone, #update_status_history_file

Constructor Details

This class inherits a constructor from Downloader

Instance Method Details

#attach_changelog_to_issues(issue_datas:, issue_jsons:) ⇒ Object



132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
# File 'lib/jirametrics/downloader_for_cloud.rb', line 132

def attach_changelog_to_issues issue_datas:, issue_jsons:
  max_results = 10_000 # The max jira accepts is 10K
  payload = {
    'issueIdsOrKeys' => issue_datas.collect(&:key),
    'maxResults' => max_results
  }
  loop do
    response = @jira_gateway.post_request(
      relative_url: '/rest/api/3/changelog/bulkfetch',
      payload: JSON.generate(payload)
    )

    response['issueChangeLogs'].each do |issue_change_log|
      issue_id = issue_change_log['issueId']
      json = issue_jsons.find { |json| json['id'] == issue_id }

      unless json['changelog']
        # If this is our first time in, there won't be a changelog section
        json['changelog'] = {
          'startAt' => 0,
          'maxResults' => max_results,
          'total' => 0,
          'histories' => []
        }
      end

      new_changes = issue_change_log['changeHistories']
      json['changelog']['total'] += new_changes.size
      json['changelog']['histories'] += new_changes
    end

    next_page_token = response['nextPageToken']
    payload['nextPageToken'] = next_page_token
    break if next_page_token.nil?
  end
end

#bulk_fetch_issues(issue_datas:, board:, in_initial_query:) ⇒ Object



88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
# File 'lib/jirametrics/downloader_for_cloud.rb', line 88

def bulk_fetch_issues issue_datas:, board:, in_initial_query:
  # We used to use the expand option to pull in the changelog directly. Unfortunately
  # that only returns the "recent" changes, not all of them. So now we get the issue
  # without changes and then make a second call for that changes. Then we insert it
  # into the raw issue as if it had been there all along.
  log "  Downloading #{issue_datas.size} issues"
  payload = {
    'fields' => ['*all'],
    'issueIdsOrKeys' => issue_datas.collect(&:key)
  }
  response = @jira_gateway.post_request(
    relative_url: '/rest/api/3/issue/bulkfetch',
    payload: JSON.generate(payload)
  )

  attach_changelog_to_issues issue_datas: issue_datas, issue_jsons: response['issues']

  response['issues'].each do |issue_json|
    issue_json['exporter'] = {
      'in_initial_query' => in_initial_query
    }
    issue = Issue.new(raw: issue_json, board: board)
    data = issue_datas.find { |d| d.key == issue.key }
    unless data
      log "  Skipping #{issue.key}: returned by Jira but key not in request (issue may have been moved)"
      next
    end
    data.up_to_date = true
    data.last_modified = issue.updated
    data.issue = issue
  end

  # Mark any unmatched requests as up_to_date to prevent infinite re-fetching.
  # This happens when Jira returns a different key (moved issue) leaving the original unmatched.
  issue_datas.each do |data|
    next if data.up_to_date

    log "  Skipping #{data.key}: not returned by Jira (issue may have been deleted or moved)"
    data.up_to_date = true
  end

  issue_datas
end


268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
# File 'lib/jirametrics/downloader_for_cloud.rb', line 268

def collect_related_issue_keys issue:, related_issue_keys:
  parent_key = issue.parent_key(project_config: @download_config.project_config)
  related_issue_keys << parent_key if parent_key

  issue.raw['fields']['subtasks']&.each do |raw_subtask|
    related_issue_keys << raw_subtask['key']
  end

  issue.raw['fields']['issuelinks']&.each do |link|
    next if link['type']['name'] == 'Cloners'

    linked = link['inwardIssue'] || link['outwardIssue']
    related_issue_keys << linked['key'] if linked
  end
end

#delete_issues_from_cache_that_are_not_in_server(issue_data_hash:, path:) ⇒ Object



249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
# File 'lib/jirametrics/downloader_for_cloud.rb', line 249

def delete_issues_from_cache_that_are_not_in_server issue_data_hash:, path:
  # The gotcha with deleted issues is that they just stop being returned in queries
  # and we have no way to know that they should be removed from our local cache.
  # With the new approach, we ask for every issue that Jira knows about (within
  # the parameters of the query) and then delete anything that's in our local cache
  # but wasn't returned.
  @file_system.foreach path do |file|
    next if file.start_with? '.'
    unless /^(?<key>\w+-\d+)-\d+\.json$/ =~ file
      raise "Unexpected filename in #{path}: #{file}"
    end
    next if issue_data_hash[key] # Still in Jira

    file_to_delete = File.join(path, file)
    log "  Removing #{file_to_delete} from local cache"
    file_system.unlink file_to_delete
  end
end

#download_board_configuration(board_id:) ⇒ Object



13
14
15
16
17
18
# File 'lib/jirametrics/downloader_for_cloud.rb', line 13

def download_board_configuration board_id:
  board = super
  location = board.raw['location']
  @project_key ||= location['key'] if location&.[]('type') == 'project'
  board
end

#download_fix_versionsObject



20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
# File 'lib/jirametrics/downloader_for_cloud.rb', line 20

def download_fix_versions
  return unless @project_key

  log "  Downloading fix versions for project #{@project_key}", both: true
  max_results = 50
  start_at = 0
  all_versions = []

  loop do
    json = @jira_gateway.call_url(
      relative_url: "/rest/api/3/project/#{@project_key}/version?" \
        "startAt=#{start_at}&maxResults=#{max_results}"
    )

    values = json['values'] || []
    all_versions.concat(values)
    break if json['isLast'] || values.empty?

    start_at += values.size
  end

  @file_system.save_json(
    json: all_versions,
    filename: File.join(@target_path, "#{file_prefix}_fix_versions.json")
  )
end

#download_issues(board:) ⇒ Object



169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
# File 'lib/jirametrics/downloader_for_cloud.rb', line 169

def download_issues board:
  log "  Downloading primary issues for board #{board.id} from #{jira_instance_type}", both: true
  path = File.join(@target_path, "#{file_prefix}_issues/")
  unless @file_system.dir_exist?(path)
    log "  Creating path #{path}"
    @file_system.mkdir(path)
  end

  filter_id = @board_id_to_filter_id[board.id]
  jql = make_jql(filter_id: filter_id)
  intercept_jql = @download_config.project_config.settings['intercept_jql']
  jql = intercept_jql.call jql if intercept_jql

  issue_data_hash = search_for_issues jql: jql, board_id: board.id, path: path

  checked_for_related = Set.new
  in_related_phase = false

  loop do
    related_issue_keys = Set.new
    stale = issue_data_hash.values.reject { |data| data.up_to_date }
    unless stale.empty?
      log_start '  Downloading more issues ' unless in_related_phase
      stale.each_slice(100) do |slice|
        slice = bulk_fetch_issues(issue_datas: slice, board: board, in_initial_query: true)
        progress_dot
        slice.each do |data|
          next unless data.issue

          @file_system.save_json(
            json: data.issue.raw, filename: data.cache_path
          )
          # Set the timestamp on the file to match the updated one so that we don't have
          # to parse the file just to find the timestamp
          @file_system.utime time: data.issue.updated, file: data.cache_path

          collect_related_issue_keys issue: data.issue, related_issue_keys: related_issue_keys
          checked_for_related << data.key
        end
      end
      end_progress unless in_related_phase
    end

    # Also scan up-to-date cached issues we haven't checked yet — they may reference
    # related issues that are not in the primary query result.
    issue_data_hash.each_value do |data|
      next if checked_for_related.include?(data.key)
      next unless @file_system.file_exist?(data.cache_path)

      checked_for_related << data.key
      raw = @file_system.load_json(data.cache_path)
      collect_related_issue_keys issue: Issue.new(raw: raw, board: board), related_issue_keys: related_issue_keys
    end

    # Remove all the ones we already have
    related_issue_keys.reject! { |key| issue_data_hash[key] }

    related_issue_keys.each do |key|
      data = DownloadIssueData.new key: key
      data.found_in_primary_query = false
      data.up_to_date = false
      data.cache_path = File.join(path, "#{key}-#{board.id}.json")
      issue_data_hash[key] = data
    end
    break if related_issue_keys.empty?

    unless in_related_phase
      in_related_phase = true
      log "  Identifying related issues (parents, subtasks, links) for board #{board.id}", both: true
      log_start '  Downloading more issues '
    end
  end

  end_progress if in_related_phase

  delete_issues_from_cache_that_are_not_in_server(
    issue_data_hash: issue_data_hash, path: path
  )
end

#jira_instance_typeObject



4
5
6
# File 'lib/jirametrics/downloader_for_cloud.rb', line 4

def jira_instance_type
  'Jira Cloud'
end

#last_modified(filename:) ⇒ Object



284
285
286
# File 'lib/jirametrics/downloader_for_cloud.rb', line 284

def last_modified filename:
  File.mtime(filename) if File.exist?(filename)
end

#runObject



8
9
10
11
# File 'lib/jirametrics/downloader_for_cloud.rb', line 8

def run
  super
  download_fix_versions
end

#search_for_issues(jql:, board_id:, path:) ⇒ Object



47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
# File 'lib/jirametrics/downloader_for_cloud.rb', line 47

def search_for_issues jql:, board_id:, path:
  log "  JQL: #{jql}"
  escaped_jql = CGI.escape jql

  hash = {}
  max_results = 5_000 # The maximum allowed by Jira
  next_page_token = nil
  issue_count = 0

  start_progress
  loop do
    relative_url = +''
    relative_url << '/rest/api/3/search/jql'
    relative_url << "?jql=#{escaped_jql}&maxResults=#{max_results}"
    relative_url << "&nextPageToken=#{next_page_token}" if next_page_token
    relative_url << '&fields=updated'

    json = @jira_gateway.call_url relative_url: relative_url
    next_page_token = json['nextPageToken']

    json['issues'].each do |i|
      key = i['key']
      data = DownloadIssueData.new key: key
      data.key = key
      data.last_modified = Time.parse i['fields']['updated']
      data.found_in_primary_query = true
      data.cache_path = File.join(path, "#{key}-#{board_id}.json")
      data.up_to_date = last_modified(filename: data.cache_path) == data.last_modified
      hash[key] = data
      issue_count += 1
    end

    progress_dot "    Found #{issue_count} issues"

    break unless next_page_token
  end
  end_progress

  hash
end