Class: Issue

Inherits:
Object
  • Object
show all
Defined in:
lib/jirametrics/issue.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(raw:, board:, timezone_offset: '+00:00') ⇒ Issue

Returns a new instance of Issue.



9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
# File 'lib/jirametrics/issue.rb', line 9

def initialize raw:, board:, timezone_offset: '+00:00'
  @raw = raw
  @timezone_offset = timezone_offset
  @subtasks = []
  @changes = []
  @board = board

  # We only check for this here because if a board isn't passed in then things will fail much
  # later and be hard to find. Let's find out early.
  raise "No board for issue #{key}" if board.nil?

  # There are cases where we create an Issue of fragments like linked issues and those won't have
  # changelogs.
  return unless @raw['changelog']

  load_history_into_changes

  # If this is an older pull of data then comments may not be there.
  load_comments_into_changes if @raw['fields']['comment']

  # It might appear that Jira already returns these in order but we've found different
  # versions of Server/Cloud return the changelog in different orders so we sort them.
  sort_changes!

  # It's possible to have a ticket created with certain things already set and therefore
  # not showing up in the change log. Create some artificial entries to capture those.
  @changes = [
    fabricate_change(field_name: 'status'),
    fabricate_change(field_name: 'priority')
  ].compact + @changes
rescue # rubocop:disable Style/RescueStandardError
  # All we're doing is adding information to the existing exception and letting it propogate up
  raise "Unable to initialize #{raw['key']}"
end

Instance Attribute Details

#boardObject (readonly)

Returns the value of attribute board.



6
7
8
# File 'lib/jirametrics/issue.rb', line 6

def board
  @board
end

#changesObject (readonly)

Returns the value of attribute changes.



6
7
8
# File 'lib/jirametrics/issue.rb', line 6

def changes
  @changes
end

#parentObject

Returns the value of attribute parent.



7
8
9
# File 'lib/jirametrics/issue.rb', line 7

def parent
  @parent
end

#rawObject (readonly)

Returns the value of attribute raw.



6
7
8
# File 'lib/jirametrics/issue.rb', line 6

def raw
  @raw
end

#subtasksObject (readonly)

Returns the value of attribute subtasks.



6
7
8
# File 'lib/jirametrics/issue.rb', line 6

def subtasks
  @subtasks
end

Instance Method Details

#<=>(other) ⇒ Object

Sort by key



575
576
577
578
579
580
581
# File 'lib/jirametrics/issue.rb', line 575

def <=> other
  /(?<project_code1>[^-]+)-(?<id1>.+)/ =~ key
  /(?<project_code2>[^-]+)-(?<id2>.+)/ =~ other.key
  comparison = project_code1 <=> project_code2
  comparison = id1 <=> id2 if comparison.zero?
  comparison
end

#all_subtask_activity_timesObject



452
453
454
455
456
457
458
# File 'lib/jirametrics/issue.rb', line 452

def all_subtask_activity_times
  subtask_activity_times = []
  @subtasks.each do |subtask|
    subtask_activity_times += subtask.changes.collect(&:time)
  end
  subtask_activity_times
end

#artificial?Boolean

It’s artificial if it wasn’t downloaded from a Jira instance.

Returns:

  • (Boolean)


570
571
572
# File 'lib/jirametrics/issue.rb', line 570

def artificial?
  @raw['exporter'].nil?
end

#assigned_toObject



225
226
227
# File 'lib/jirametrics/issue.rb', line 225

def assigned_to
  @raw['fields']&.[]('assignee')&.[]('displayName')
end

#authorObject



54
# File 'lib/jirametrics/issue.rb', line 54

def author = @raw['fields']['creator']&.[]('displayName') || ''

#blocked_on_date?(date, end_time:) ⇒ Boolean

Returns:

  • (Boolean)


235
236
237
# File 'lib/jirametrics/issue.rb', line 235

def blocked_on_date? date, end_time:
  (blocked_stalled_by_date date_range: date..date, chart_end_time: end_time)[date].blocked?
end

#blocked_stalled_by_date(date_range:, chart_end_time:, settings: nil) ⇒ Object

For any day in the day range… If the issue was blocked at any point in this day, the whole day is blocked. If the issue was active at any point in this day, the whole day is active If the day was stalled for the entire day then it’s stalled If there was no activity at all on this day then the last change from the previous day carries over



244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
# File 'lib/jirametrics/issue.rb', line 244

def blocked_stalled_by_date date_range:, chart_end_time:, settings: nil
  results = {}
  current_date = nil
  blocked_stalled_changes = blocked_stalled_changes(end_time: chart_end_time, settings: settings)
  blocked_stalled_changes.each do |change|
    current_date = change.time.to_date

    winning_change, _last_change = results[current_date]
    if winning_change.nil? ||
      change.blocked? ||
      (change.active? && (winning_change.active? || winning_change.stalled?)) ||
      (change.stalled? && winning_change.stalled?)

      winning_change = change
    end

    results[current_date] = [winning_change, change]
  end

  last_populated_date = nil
  (results.keys.min..results.keys.max).each do |date|
    if results.key? date
      last_populated_date = date
    else
      _winner, last = results[last_populated_date]
      results[date] = [last, last]
    end
  end
  results = results.transform_values(&:first)

  # The requested date range may span outside the actual changes we find in the changelog
  date_of_first_change = blocked_stalled_changes[0].time.to_date
  date_of_last_change = blocked_stalled_changes[-1].time.to_date
  date_range.each do |date|
    results[date] = blocked_stalled_changes[0] if date < date_of_first_change
    results[date] = blocked_stalled_changes[-1] if date > date_of_last_change
  end

  # To make the code simpler, we've been accumulating data for every date. Now remove anything
  # that isn't in the requested date_range
  results.select! { |date, _value| date_range.include? date }

  results
end

#blocked_stalled_changes(end_time:, settings: nil) ⇒ Object



289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
# File 'lib/jirametrics/issue.rb', line 289

def blocked_stalled_changes end_time:, settings: nil
  settings ||= @board.project_config.settings

  blocked_statuses = settings['blocked_statuses']
  stalled_statuses = settings['stalled_statuses']
  unless blocked_statuses.is_a?(Array) && stalled_statuses.is_a?(Array)
    raise "blocked_statuses(#{blocked_statuses.inspect}) and " \
      "stalled_statuses(#{stalled_statuses.inspect}) must both be arrays"
  end

  blocked_link_texts = settings['blocked_link_text']
  stalled_threshold = settings['stalled_threshold_days']
  flagged_means_blocked = !!settings['flagged_means_blocked'] # rubocop:disable Style/DoubleNegation

  blocking_issue_keys = []

  result = []
  previous_was_active = false # Must start as false so that the creation will insert an :active
  previous_change_time = created

  blocking_status = nil
  flag = nil

  # This mock change is to force the writing of one last entry at the end of the time range.
  # By doing this, we're able to eliminate a lot of duplicated code in charts.
  mock_change = ChangeItem.new time: end_time, author: '', artificial: true, raw: { 'field' => '' }

  (changes + [mock_change]).each do |change|
    previous_was_active = false if check_for_stalled(
      change_time: change.time,
      previous_change_time: previous_change_time,
      stalled_threshold: stalled_threshold,
      blocking_stalled_changes: result
    )

    if change.flagged? && flagged_means_blocked
      flag = change.value
      flag = nil if change.value == ''
    elsif change.status?
      blocking_status = nil
      if blocked_statuses.include?(change.value) || stalled_statuses.include?(change.value)
        blocking_status = change.value
      end
    elsif change.link?
      # Example: "This issue is satisfied by ANON-30465"
      unless /^This issue (?<link_text>.+) (?<issue_key>.+)$/ =~ (change.value || change.old_value)
        puts "Issue(#{key}) Can't parse link text: #{change.value || change.old_value}"
        next
      end

      if blocked_link_texts.include? link_text
        if change.value
          blocking_issue_keys << issue_key
        else
          blocking_issue_keys.delete issue_key
        end
      end
    end

    new_change = BlockedStalledChange.new(
      flagged: flag,
      status: blocking_status,
      status_is_blocking: blocking_status.nil? || blocked_statuses.include?(blocking_status),
      blocking_issue_keys: (blocking_issue_keys.empty? ? nil : blocking_issue_keys.dup),
      time: change.time
    )

    # We don't want to dump two actives in a row as that would just be noise. Unless this is
    # the mock change, which we always want to dump
    result << new_change if !new_change.active? || !previous_was_active || change == mock_change

    previous_was_active = new_change.active?
    previous_change_time = change.time
  end

  if result.size >= 2
    # The existence of the mock entry will mess with the stalled count as it will wake everything
    # back up. This hack will clean up appropriately.
    hack = result.pop
    result << BlockedStalledChange.new(
      flagged: hack.flag,
      status: hack.status,
      status_is_blocking: hack.status_is_blocking,
      blocking_issue_keys: hack.blocking_issue_keys,
      time: hack.time,
      stalled_days: result[-1].stalled_days
    )
  end

  result
end

#check_for_stalled(change_time:, previous_change_time:, stalled_threshold:, blocking_stalled_changes:) ⇒ Object



381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
# File 'lib/jirametrics/issue.rb', line 381

def check_for_stalled change_time:, previous_change_time:, stalled_threshold:, blocking_stalled_changes:
  stalled_threshold_seconds = stalled_threshold * 60 * 60 * 24

  # The most common case will be nothing to split so quick escape.
  return false if (change_time - previous_change_time).to_i < stalled_threshold_seconds

  # If the last identified change was blocked then it doesn't matter now long we've waited, we're still blocked.
  return false if blocking_stalled_changes[-1]&.blocked?

  list = [previous_change_time..change_time]
  all_subtask_activity_times.each do |time|
    matching_range = list.find { |range| time >= range.begin && time <= range.end }
    next unless matching_range

    list.delete matching_range
    list << ((matching_range.begin)..time)
    list << (time..(matching_range.end))
  end

  inserted_stalled = false

  list.sort_by(&:begin).each do |range|
    seconds = (range.end - range.begin).to_i
    next if seconds < stalled_threshold_seconds

    an_hour_later = range.begin + (60 * 60)
    blocking_stalled_changes << BlockedStalledChange.new(stalled_days: seconds / (24 * 60 * 60), time: an_hour_later)
    inserted_stalled = true
  end
  inserted_stalled
end

#component_namesObject



81
82
83
# File 'lib/jirametrics/issue.rb', line 81

def component_names
  @raw['fields']['components']&.collect { |component| component['name'] } || []
end

#createdObject



208
209
210
211
# File 'lib/jirametrics/issue.rb', line 208

def created
  # This nil check shouldn't be necessary and yet we've seen one case where it was.
  parse_time @raw['fields']['created'] if @raw['fields']['created']
end

#currently_in_status(*status_names) ⇒ Object

Are we currently in this status? If yes, then return the most recent status change.



149
150
151
152
153
# File 'lib/jirametrics/issue.rb', line 149

def currently_in_status *status_names
  change = most_recent_status_change

  change if change.current_status_matches(*status_names)
end

#currently_in_status_category(*category_names) ⇒ Object

Are we currently in this status category? If yes, then return the most recent status change.



156
157
158
159
160
161
162
163
# File 'lib/jirametrics/issue.rb', line 156

def currently_in_status_category *category_names
  category_ids = find_status_category_ids_by_names category_names

  change = most_recent_status_change

  status = find_or_create_status id: change.value_id, name: change.value
  change if status && category_ids.include?(status.category.id)
end

#discard_changes_before(cutoff_time) ⇒ Object



583
584
585
586
587
588
589
590
591
592
593
594
595
# File 'lib/jirametrics/issue.rb', line 583

def discard_changes_before cutoff_time
  rejected_any = false
  @changes.reject! do |change|
    reject = change.status? && change.time <= cutoff_time && change.artificial? == false
    if reject
      (@discarded_changes ||= []) << change
      rejected_any = true
    end
    reject
  end

  (@discarded_change_times ||= []) << cutoff_time if rejected_any
end

#done?Boolean

Returns:

  • (Boolean)


660
661
662
663
664
665
666
667
668
669
# File 'lib/jirametrics/issue.rb', line 660

def done?
  if artificial? || board.cycletime.nil?
    # This was probably loaded as a linked issue, which means we don't know what board it really
    # belonged to. The best we can do is look at the status category. This case should be rare but
    # it can happen.
    status.category.name == 'Done'
  else
    board.cycletime.done? self
  end
end

#due_dateObject



67
68
69
70
# File 'lib/jirametrics/issue.rb', line 67

def due_date
  text = @raw['fields']['duedate']
  text.nil? ? nil : Date.parse(text)
end

#dumpObject



597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
# File 'lib/jirametrics/issue.rb', line 597

def dump
  result = +''
  result << "#{key} (#{type}): #{compact_text summary, 200}\n"

  assignee = raw['fields']['assignee']
  result << "  [assignee] #{assignee['name'].inspect} <#{assignee['emailAddress']}>\n" unless assignee.nil?

  raw['fields']['issuelinks']&.each do |link|
    result << "  [link] #{link['type']['outward']} #{link['outwardIssue']['key']}\n" if link['outwardIssue']
    result << "  [link] #{link['type']['inward']} #{link['inwardIssue']['key']}\n" if link['inwardIssue']
  end
  history = [] # time, type, detail

  started_at, stopped_at = board.cycletime.started_stopped_times(self)
  history << [started_at, nil, '↓↓↓↓ Started here ↓↓↓↓', true] if started_at
  history << [stopped_at, nil, '↑↑↑↑ Finished here ↑↑↑↑', true] if stopped_at

  @discarded_change_times&.each do |time|
    history << [time, nil, '↑↑↑↑ Changes discarded ↑↑↑↑', true]
  end

  (changes + (@discarded_changes || [])).each do |change|
    value = change.value
    old_value = change.old_value

    message = +''
    message << "#{compact_text(old_value).inspect} -> " unless old_value.nil? || old_value.empty?
    message << compact_text(value).inspect
    if change.artificial?
      message << ' (Artificial entry)' if change.artificial?
    else
      message << " (Author: #{change.author})"
    end
    history << [change.time, change.field, message, change.artificial?]
  end

  result << "  History:\n"
  type_width = history.collect { |_time, type, _detail, _artificial| type&.length || 0 }.max
  history.sort! do |a, b|
    if a[0] == b[0]
      if a[1].nil?
        1
      elsif b[1].nil?
        -1
      else
        a[1] <=> b[1]
      end
    else
      a[0] <=> b[0]
    end
  end
  history.each do |time, type, detail, _artificial|
    if type.nil?
      type = '-' * type_width
    else
      type = (' ' * (type_width - type.length)) << type
    end
    result << "    #{time.strftime '%Y-%m-%d %H:%M:%S %z'} [#{type}] #{detail}\n"
  end

  result
end

#expedited?Boolean

Returns:

  • (Boolean)


460
461
462
463
464
465
466
467
468
# File 'lib/jirametrics/issue.rb', line 460

def expedited?
  return false unless @board&.project_config

  names = @board.project_config.settings['expedited_priority_names']
  return false unless names

  current_priority = raw['fields']['priority']&.[]('name')
  names.include? current_priority
end

#expedited_on_date?(date) ⇒ Boolean

Returns:

  • (Boolean)


470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
# File 'lib/jirametrics/issue.rb', line 470

def expedited_on_date? date
  expedited_start = nil
  return false unless @board&.project_config

  expedited_names = @board.project_config.settings['expedited_priority_names']

  changes.each do |change|
    next unless change.priority?

    if expedited_names.include? change.value
      expedited_start = change.time.to_date if expedited_start.nil?
    else
      return true if expedited_start && (expedited_start..change.time.to_date).cover?(date)

      expedited_start = nil
    end
  end

  return false if expedited_start.nil?

  expedited_start <= date
end

#find_or_create_status(id:, name:) ⇒ Object



165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
# File 'lib/jirametrics/issue.rb', line 165

def find_or_create_status id:, name:
  status = board.possible_statuses.find_by_id(id)

  unless status
    # Have to pull this list before the call to fabricate or else the warning will incorrectly
    # list this status as one it actually found
    found_statuses = board.possible_statuses.to_s

    status = board.possible_statuses.fabricate_status_for id: id, name: name

    message = +'The history for issue '
    message << key
    message << ' references the status ('
    message << "#{name.inspect}:#{id.inspect}"
    message << ') that can\'t be found. We are guessing that this belongs to the '
    message << status.category.to_s
    message << ' status category but that may be wrong. See https://jirametrics.org/faq/#q1 for more '
    message << 'details on defining statuses.'
    board.project_config.file_system.warning message, more: "The statuses we did find are: #{found_statuses}"
  end

  status
end

#first_resolutionObject



217
218
219
# File 'lib/jirametrics/issue.rb', line 217

def first_resolution
  @changes.find { |change| change.resolution? }
end

#first_status_change_after_createdObject



189
190
191
# File 'lib/jirametrics/issue.rb', line 189

def first_status_change_after_created
  status_changes.find { |change| change.artificial? == false }
end

#first_time_in_or_right_of_column(column_name) ⇒ Object



93
94
95
# File 'lib/jirametrics/issue.rb', line 93

def first_time_in_or_right_of_column column_name
  first_time_in_status(*board.status_ids_in_or_right_of_column(column_name))
end

#first_time_in_status(*status_names) ⇒ Object



85
86
87
# File 'lib/jirametrics/issue.rb', line 85

def first_time_in_status *status_names
  @changes.find { |change| change.current_status_matches(*status_names) }
end

#first_time_in_status_category(*category_names) ⇒ Object



193
194
195
196
197
198
199
200
201
202
# File 'lib/jirametrics/issue.rb', line 193

def first_time_in_status_category *category_names
  category_ids = find_status_category_ids_by_names category_names

  status_changes.each do |change|
    to_status = find_or_create_status(id: change.value_id, name: change.value)
    id = to_status.category.id
    return change if category_ids.include? id
  end
  nil
end

#first_time_label_added(*labels) ⇒ Object



97
98
99
100
101
102
103
104
105
# File 'lib/jirametrics/issue.rb', line 97

def first_time_label_added *labels
  @changes.each do |change|
    next unless change.labels?

    change_labels = change.value.split
    return change if change_labels.any? { |l| labels.include?(l) }
  end
  nil
end

#first_time_not_in_status(*status_names) ⇒ Object



89
90
91
# File 'lib/jirametrics/issue.rb', line 89

def first_time_not_in_status *status_names
  @changes.find { |change| change.status? && status_names.include?(change.value) == false }
end

#fix_versionsObject



518
519
520
521
522
523
524
525
# File 'lib/jirametrics/issue.rb', line 518

def fix_versions
  if @fix_versions.nil?
    @fix_versions = @raw['fields']['fixVersions']&.collect do |fix_version|
      FixVersion.new fix_version
    end || []
  end
  @fix_versions
end

#flow_efficiency_numbers(end_time:, settings: @board.project_config.settings) ⇒ Object

return [number of active seconds, total seconds] that this issue had up to the end_time. It does not include data before issue start or after issue end



415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
# File 'lib/jirametrics/issue.rb', line 415

def flow_efficiency_numbers end_time:, settings: @board.project_config.settings
  issue_start, issue_stop = @board.cycletime.started_stopped_times(self)
  return [0.0, 0.0] if !issue_start || issue_start > end_time

  value_add_time = 0.0
  end_time = issue_stop if issue_stop && issue_stop < end_time

  active_start = nil
  blocked_stalled_changes(end_time: end_time, settings: settings).each_with_index do |change, index|
    break if change.time > end_time

    if index.zero?
      active_start = change.time if change.active?
      next
    end

    # Already active and we just got another active.
    next if active_start && change.active?

    if change.active?
      active_start = change.time
    elsif active_start && change.time >= issue_start
      # Not active now but we have been. Record the active time.
      change_delta = change.time - [issue_start, active_start].max
      value_add_time += change_delta
      active_start = nil
    end
  end

  if active_start
    change_delta = end_time - [issue_start, active_start].max
    value_add_time += change_delta if change_delta.positive?
  end

  [value_add_time, end_time - issue_start]
end

#in_initial_query?Boolean

Returns:

  • (Boolean)


565
566
567
# File 'lib/jirametrics/issue.rb', line 565

def in_initial_query?
  @raw['exporter'].nil? || @raw['exporter']['in_initial_query']
end

#inspectObject

Many test failures are simply unreadable because the default inspect on this class goes on for pages. Shorten it up.



231
232
233
# File 'lib/jirametrics/issue.rb', line 231

def inspect
  "Issue(#{key.inspect})"
end


509
510
511
512
513
514
515
516
# File 'lib/jirametrics/issue.rb', line 509

def issue_links
  if @issue_links.nil?
    @issue_links = @raw['fields']['issuelinks']&.collect do |issue_link|
      IssueLink.new origin: self, raw: issue_link
    end || []
  end
  @issue_links
end

#keyObject



44
# File 'lib/jirametrics/issue.rb', line 44

def key = @raw['key']

#key_as_iObject



77
78
79
# File 'lib/jirametrics/issue.rb', line 77

def key_as_i
  key =~ /-(\d+)$/ ? $1.to_i : 0
end

#labelsObject



52
# File 'lib/jirametrics/issue.rb', line 52

def labels = @raw['fields']['labels'] || []

#last_activity(now: Time.now) ⇒ Object

Return the last time there was any activity on this ticket. Starting from “now” and going backwards Returns nil if there was no activity before that time.



495
496
497
498
499
500
501
502
503
504
505
506
507
# File 'lib/jirametrics/issue.rb', line 495

def last_activity now: Time.now
  result = @changes.reverse.find { |change| change.time <= now }&.time

  # The only condition where this could be nil is if "now" is before creation
  return nil if result.nil?

  @subtasks.each do |subtask|
    subtask_last_activity = subtask.last_activity now: now
    result = subtask_last_activity if subtask_last_activity && subtask_last_activity > result
  end

  result
end

#last_resolutionObject



221
222
223
# File 'lib/jirametrics/issue.rb', line 221

def last_resolution
  @changes.reverse.find { |change| change.resolution? }
end

#looks_like_issue_key?(key) ⇒ Boolean

Returns:

  • (Boolean)


527
528
529
# File 'lib/jirametrics/issue.rb', line 527

def looks_like_issue_key? key
  !!(key.is_a?(String) && key =~ /^[^-]+-\d+$/)
end

#most_recent_status_changeObject



143
144
145
146
# File 'lib/jirametrics/issue.rb', line 143

def most_recent_status_change
  # We artificially insert a status change to represent creation so by definition there will always be at least one.
  changes.reverse.find { |change| change.status? }
end

#parent_key(project_config: @board.project_config) ⇒ Object



531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
# File 'lib/jirametrics/issue.rb', line 531

def parent_key project_config: @board.project_config
  # Although Atlassian is trying to standardize on one way to determine the parent, today it's a mess.
  # We try a variety of ways to get the parent and hopefully one of them will work. See this link:
  # https://community.developer.atlassian.com/t/deprecation-of-the-epic-link-parent-link-and-other-related-fields-in-rest-apis-and-webhooks/54048

  fields = @raw['fields']

  # At some point in the future, this will be the only way to retrieve the parent so we try this first.
  parent = fields['parent']&.[]('key')

  # The epic field
  parent = fields['epic']&.[]('key') if parent.nil?

  # Otherwise the parent link will be stored in one of the custom fields. We've seen different custom fields
  # used for parent_link vs epic_link so we have to support more than one.
  if parent.nil? && project_config
    custom_field_names = project_config.settings['customfield_parent_links']
    custom_field_names = [custom_field_names] if custom_field_names.is_a? String

    custom_field_names&.each do |field_name|
      parent = fields[field_name]
      next if parent.nil?
      break if looks_like_issue_key? parent

      project_config.file_system.log(
        "Custom field #{field_name.inspect} should point to a parent id but found #{parent.inspect}"
      )
      parent = nil
    end
  end

  parent
end

#parse_time(text) ⇒ Object



204
205
206
# File 'lib/jirametrics/issue.rb', line 204

def parse_time text
  Time.parse(text).getlocal(@timezone_offset)
end

#resolutionObject



56
# File 'lib/jirametrics/issue.rb', line 56

def resolution = @raw['fields']['resolution']&.[]('name')

#statusObject



58
59
60
61
# File 'lib/jirametrics/issue.rb', line 58

def status
  @status = Status.from_raw(@raw['fields']['status']) unless @status
  @status
end

#status=(status) ⇒ Object



63
64
65
# File 'lib/jirametrics/issue.rb', line 63

def status= status
  @status = status
end

#status_changesObject



671
672
673
# File 'lib/jirametrics/issue.rb', line 671

def status_changes
  @changes.select { |change| change.status? }
end

#still_in_or_right_of_column(column_name) ⇒ Object



107
108
109
# File 'lib/jirametrics/issue.rb', line 107

def still_in_or_right_of_column column_name
  still_in_status(*board.status_ids_in_or_right_of_column(column_name))
end

#still_in_status(*status_names) ⇒ Object

If it ever entered one of these statuses and it’s still there then what was the last time it entered



127
128
129
130
131
# File 'lib/jirametrics/issue.rb', line 127

def still_in_status *status_names
  still_in do |change|
    status_names.include?(change.value) || status_names.include?(change.value_id)
  end
end

#still_in_status_category(*category_names) ⇒ Object

If it ever entered one of these categories and it’s still there then what was the last time it entered



134
135
136
137
138
139
140
141
# File 'lib/jirametrics/issue.rb', line 134

def still_in_status_category *category_names
  category_ids = find_status_category_ids_by_names category_names

  still_in do |change|
    status = find_or_create_status id: change.value_id, name: change.value
    category_ids.include? status.category.id
  end
end

#summaryObject



50
# File 'lib/jirametrics/issue.rb', line 50

def summary = @raw['fields']['summary']

#typeObject



46
# File 'lib/jirametrics/issue.rb', line 46

def type = @raw['fields']['issuetype']['name']

#type_icon_urlObject



48
# File 'lib/jirametrics/issue.rb', line 48

def type_icon_url = @raw['fields']['issuetype']['iconUrl']

#updatedObject



213
214
215
# File 'lib/jirametrics/issue.rb', line 213

def updated
  parse_time @raw['fields']['updated']
end

#urlObject



72
73
74
75
# File 'lib/jirametrics/issue.rb', line 72

def url
  # Strangely, the URL isn't anywhere in the returned data so we have to fabricate it.
  "#{@board.server_url_prefix}/browse/#{key}"
end