Class: Issue::TrackedSprint
- Inherits:
-
Data
- Object
- Data
- Issue::TrackedSprint
- Defined in:
- lib/jirametrics/issue.rb
Overview
Like SprintMembership but also tracks when this issue was added, used while pairing sprint entries and exits in #sprint_entry_events.
Instance Attribute Summary collapse
-
#add_time ⇒ Object
readonly
Returns the value of attribute add_time.
-
#change ⇒ Object
readonly
Returns the value of attribute change.
-
#sprint_id ⇒ Object
readonly
Returns the value of attribute sprint_id.
-
#sprint_start ⇒ Object
readonly
Returns the value of attribute sprint_start.
Instance Attribute Details
#add_time ⇒ Object (readonly)
Returns the value of attribute add_time
242 243 244 |
# File 'lib/jirametrics/issue.rb', line 242 def add_time @add_time end |
#change ⇒ Object (readonly)
Returns the value of attribute change
242 243 244 |
# File 'lib/jirametrics/issue.rb', line 242 def change @change end |
#sprint_id ⇒ Object (readonly)
Returns the value of attribute sprint_id
242 243 244 |
# File 'lib/jirametrics/issue.rb', line 242 def sprint_id @sprint_id end |
#sprint_start ⇒ Object (readonly)
Returns the value of attribute sprint_start
242 243 244 |
# File 'lib/jirametrics/issue.rb', line 242 def sprint_start @sprint_start end |